class PrefixTuning(nn.Module):
    def __init__(self, model, num_layer, embd_dim, device, mid_dim=None, preseqlen=100, prefix_dropout=0.0):
        super(PrefixTuning, self).__init__()
        self.device = device
        self.match_n_layer = num_layer
        self.n_embd = embd_dim
        self.mid_dim = mid_dim if mid_dim is not None else embd_dim * 4
        self.preseqlen = preseqlen
        self.prefix_dropout = prefix_dropout
        self.model = model
        print('PrefixTuning')
        print('preseqlen is {}, optimizing the prefix directly'.format(self.preseqlen))
        self.input_tokens = torch.arange(self.preseqlen).long()
        self.wte = nn.Embedding(self.preseqlen, self.n_embd)
        # multiply by 2 because we need to split to kv pair in the end
        self.control_trans = nn.Sequential(
            nn.Linear(self.n_embd, self.mid_dim),
            nn.Tanh(),
            nn.Linear(self.mid_dim, self.match_n_layer * self.n_embd))
        self.dropout = nn.Dropout(self.prefix_dropout)
        # freeze all model params so that only prefixes are trained!!
        for param in model.parameters():
            param.requires_grad = False
    
    def get_prompt(self, bsz):
        prefix_tokens = self.input_tokens.unsqueeze(0).expand(bsz, -1).to(self.device)
        prefix_embeddings = self.wte(prefix_tokens)
        prefixes = self.control_trans(prefix_embeddings) #bsz, seqlen, layer*emb
        bsz, seqlen, _ = prefixes.shape
        prefixes = prefixes.view(bsz, seqlen, self.match_n_layer , self.n_embd)
        prefixes = self.dropout(prefixes)
        # reorder and split to kv pairs
        prefixes = prefixes.permute([2, 0, 1, 3])
        new_mems = [prefix_embeddings]
        for layer_mem in prefixes:
            new_mems.append(layer_mem)
        return new_mems

    def forward(self,  input_ids, position_ids, attention_mask, *mems):
        prefixes = self.get_prompt(bsz=input_ids.shape[0]) # num_layer * bsz * seq_len * hidden_dim
        if mems:
            new_mems = torch.cat([prefixes, mems], dim=2)
        else:
            new_mems = prefixes
        return self.model(input_ids, position_ids, attention_mask, *new_mems)