#!/bin/bash

CHECKPOINT_PATH="./checkpoint_model"
PREFIX_SCRIPT_PATH=$1
MPSIZE=1
NLAYERS=32
NHIDDEN=2560
NATT=32
MAXSEQLEN=1024
MASTER_PORT=$(shuf -n 1 -i 10000-65535)

#SAMPLING ARGS
TEMP=0.9
#If TOPK/TOPP are 0 it defaults to greedy sampling, top-k will also override top-p
TOPK=0
TOPP=0.9

script_path=$(realpath $0)
script_dir=$(dirname $script_path)

config_json="$script_dir/ds_config.json"
OPTIONS_NCCL="NCCL_DEBUG=info NCCL_IB_DISABLE=0 NCCL_NET_GDR_LEVEL=2"
HOST_FILE_PATH="/desktop/hostfile" 
STATE_DICT_PATH="./transformer-xl-cmrc-prefix-step-4000/mp_rank_00_model_states.pt"
RUN_OPTIONS="
       --MASTER_PORT $MASTER_PORT
       --model-parallel-size $MPSIZE \
       --num-layers $NLAYERS \
       --hidden-size $NHIDDEN \
       --load $CHECKPOINT_PATH \
       --num-attention-heads $NATT \
       --max-position-embeddings 1024 \
       --tokenizer-type ChineseSPTokenizer \
       --fp16 \
       --cache-dir cache \
       --out-seq-length $MAXSEQLEN \
       --seq-length 512 \
       --mem-length 256 \
       --transformer-xl \
       --temperature $TEMP \
       --top_k $TOPK \
       --top_p $TOPP \
       --batch_size 1 \
       --deepspeed \
       --deepspeed-activation-checkpointing \
       --deepspeed_config './scripts/ds_config_2.9B_finetune.json' \
       --save_dir "./checkpoint_model" \
       --preseqlen 200 \
       --no_load_lr_scheduler \
       --no_load_optim \
       --finetune \
"
#       --load_sd $STATE_DICT_PATH \

RUN_CMD="$OPTIONS_NCCL deepspeed --include="localhost:1,2,3,4" --hostfile=$HOST_FILE_PATH $PREFIX_SCRIPT_PATH ${RUN_OPTIONS}" 
echo ${RUN_CMD}
eval ${RUN_CMD}

